#include "Classcode/Thresholding.hh"
#include "Classcode/AdaBoost.hh"

using namespace CLASSCODE;

typedef Thresholding WeakClassifier;
typedef AdaBoost<WeakClassifier> BoostClassifier;

GP_Vector randomvec(uint len)
{
  static gsl_rng *rng = 0;
  if(rng == 0)
    rng = gsl_rng_alloc(gsl_rng_default);
  
  //struct timeval tv;
  //gettimeofday(&tv, 0);
  //gsl_rng_set (rng, tv.tv_usec);

  GP_Vector smplvec(len);
  for(uint i=0; i<len; ++i){
    double smpl = gsl_rng_uniform_int(rng, RAND_MAX);
    smplvec[i] = smpl / RAND_MAX;
  }

  //gsl_rng_free(rng);

  return smplvec;
}

BEGIN_PROGRAM(argc, argv)
{
  std::vector<GP_Vector> feat(2000);
  
  for(  uint k=0; k< feat.size(); k++){
    feat[k] = 10 * randomvec(2);
  }

  std::vector<int> labels(feat.size());

  for(uint i=0; i<labels.size(); i++){
    //double d = GP_Vector::Random(1)[0];
    double r = 3;

    GP_Vector vec(2);
    vec[0] = 3; vec[1] = 4;
    if((feat[i] - vec).Norm() < r)
      labels[i] = -1;      
      //labels[i] = 0;      

    else
      labels[i] = 1;
  }

  WRITE_FILE(trfile, "train.dat");
  for(uint i=0; i<feat.size(); i++)
    if(labels[i] == -1)
      trfile << feat[i] << std::endl;

  trfile << std::endl << std::endl;
  for(uint i=0; i<feat.size(); i++)
    if(labels[i] == 1)
      trfile << feat[i] << std::endl;
  trfile.close();

  if(argc != 2){
    throw GP_EXCEPTION("Test requires number of stumps as argument!");
  }

  BoostClassifier ada(atoi(argv[1]), 2);
  BoostClassifier::DataSet data;
  data.Append(feat, labels);
  ada.Train(data);

  std::cout << "learning done" << std::endl;
  //double class_uncert = ada.GetUCC();
  //std::cout << "classification uncertainty " << class_uncert << std::endl;

  std::vector<GP_Vector> test_feat(1000);
  std::vector<int> test_labels(test_feat.size());
  for(uint i=0; i<test_feat.size(); i++){
    test_feat[i] = randomvec(2);
    test_feat[i][0] *= 16;
    test_feat[i][1] *= 5;
  }  
  for(uint i=0; i<test_labels.size(); i++){
    test_labels[i] = (ada.Predict(test_feat[i]) > 0.5 ? 1 : -1);
  }

  WRITE_FILE(tefile, "test.dat");
  for(uint i=0; i<test_labels.size(); i++)
    if(test_labels[i] == -1)
      tefile << test_feat[i] << std::endl;

  tefile << std::endl << std::endl;
  for(uint i=0; i<test_labels.size(); i++)
    if(test_labels[i] == 1)
      tefile << test_feat[i] << std::endl;
  tefile.close();

  std::cout << "training error:  " << ada.GetTrainingError() << std::endl;

}END_PROGRAM
